from .models import GeneralInfoConfiguration
import os
import subprocess
import re
import shutil


def server_information_system_monitor():
    statistics = {}
    matcher = re.compile('\d+')
    # Top command on mac displays and updates sorted information about processes.

    top_command = subprocess.run(['top', '-l 1', '-n 0'], stdout=subprocess.PIPE).stdout.decode('utf-8').split('\n')

    # Get Physical and Logical CPU Count
    physical_and_logical_cpu_count = os.cpu_count()
    statistics['physical_and_logical_cpu_count'] = physical_and_logical_cpu_count
    """
    # Load average 
    # This is the average system load calculated over a given period of time of 1, 5 and 15 minutes.
    # In our case, we will show the load average over a period of 15 minutes.

    # The numbers returned by os.getloadavg() only make sense if
    # related to the number of CPU cores installed on the system.

    # Here we are converting the load average into percentage. The higher the percentage the higher the load
    """

    cpu_load = [x / os.cpu_count() * 100 for x in os.getloadavg()][-1]
    statistics['cpu_load'] = round(cpu_load)

    # Memory usage
    total_ram = subprocess.run(['sysctl', 'hw.memsize'], stdout=subprocess.PIPE).stdout.decode('utf-8')
    vm = subprocess.Popen(['vm_stat'], stdout=subprocess.PIPE).communicate()[0].decode('utf-8')
    vmLines = vm.split('\n')

    wired_memory = (int(matcher.search(vmLines[6]).group()) * 4096) / 1024 ** 3
    free_memory = (int(matcher.search(vmLines[1]).group()) * 4096) / 1024 ** 3
    active_memory = (int(matcher.search(vmLines[2]).group()) * 4096) / 1024 ** 3
    inactive_memory = (int(matcher.search(vmLines[3]).group()) * 4096) / 1024 ** 3

    # Used memory = wired_memory + inactive + active

    statistics['ram'] = dict({
        'total_ram': int(matcher.search(total_ram).group()) / 1024 ** 3,
        'used_ram': round(wired_memory + active_memory + inactive_memory, 2),
    })

    # Disk usage
    # Get total disk size, used disk space, and free disk

    total, used, free = shutil.disk_usage("/")

    # Number of Read and write operations
    # from the top command, the read written result will be as follows
    # 'Disks: XXXXXX/xxG read, XXXX/xxG written.'
    # we thus need to extract the read and written from this.
    read_written = top_command[9].split(':')[1].split(',')
    read = read_written[0].split(' ')[1]
    written = read_written[1].split(' ')[1]

    statistics['disk'] = dict(
        {
            'total_disk_space': round(total / 1024 ** 3, 1),
            'used_disk_space': round(used / 1024 ** 3, 1),
            'free_disk_space': round(free / 1024 ** 3, 1),
            'read_write': {
                'read': read,
                'written': written
            }
        }
    )
    return statistics


def server_information_ping(
        ip: str
):
    statistics = {}
    # Network latency
    """
    Here we will ping google at an interval of five seconds for five times and record the
    min response time, average response time, and the max response time. 
    """
    ping_result = subprocess.run(['ping', '-i 5', '-c 5', ip], stdout=subprocess.PIPE).stdout.decode(
        'utf-8').split('\n')
    min, avg, max = ping_result[-2].split('=')[-1].split('/')[:3]
    print(ping_result[0])
    print(ping_result[1])
    print(ping_result[2])
    print(ping_result[3])
    print(ping_result[4])
    print(ping_result[5])
    print(ping_result[7])
    print(ping_result[8])
    statistics['network_latency'] = dict(
        {
            'min': min.strip(),
            'avg': avg.strip(),
            'max': max.strip(),
        }
    )
    print(statistics)


def create_update_general_info(
        logo=None,
        favicon=None,
        address: str = None,
        contact_emails: str = None,
        attention_schedule: str = None,
        facebook: str = None,
        whatsaap: str = None,
        instagram: str = None,
        linkedin: str = None,
        youtube: str = None,
        twitter: str = None,
        application_name: str = '',
) -> GeneralInfoConfiguration:
    general_info = GeneralInfoConfiguration.objects.all()
    if not general_info.exists():
        general_info = GeneralInfoConfiguration()
    else:
        general_info = GeneralInfoConfiguration.objects.first()
    if logo is not None:
        general_info.logo = logo
    if favicon is not None:
        general_info.favicon = favicon
    general_info.address = address
    general_info.contact_emails = contact_emails
    general_info.attention_schedule = attention_schedule
    general_info.facebook = facebook
    general_info.whatsaap = whatsaap
    general_info.instagram = instagram
    general_info.linkedin = linkedin
    general_info.youtube = youtube
    general_info.twitter = twitter
    general_info.application_name = application_name
    general_info.save()
    return GeneralInfoConfiguration.objects.first()



# Generated by Django 3.0.5 on 2020-04-02 15:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users_extended', '0002_auto_20200402_1300'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserAditionalPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': (('list_user', 'Can list user'), ('change_password_user', 'Can change user password'), ('detail_user', 'Can detail user'), ('make_superuser_user', 'Can make user superuser'), ('make_staff_user', 'Can make user staff'), ('make_active_user', 'Can make user active')),
                'managed': False,
            },
        ),
    ]



from django.db.models.signals import pre_delete, post_init, post_save
from django.dispatch import receiver

from .models import GeneralInfoConfiguration


@receiver(pre_delete, sender=GeneralInfoConfiguration)
def logo_pre_delete(sender, instance, **kwargs):
    instance.logo.delete(False)


@receiver(post_init, sender=GeneralInfoConfiguration)
def backup_logo_path(sender, instance, **kwargs):
    instance._current_logo = instance.logo


@receiver(post_save, sender=GeneralInfoConfiguration)
def delete_logo(sender, instance, **kwargs):
    if hasattr(instance, '_current_logo'):
        if instance._current_logo != instance.logo:
            instance._current_logo.delete(save=False)


@receiver(pre_delete, sender=GeneralInfoConfiguration)
def favicon_pre_delete(sender, instance, **kwargs):
    instance.favicon.delete(False)


@receiver(post_init, sender=GeneralInfoConfiguration)
def backup_favicon_path(sender, instance, **kwargs):
    instance._current_favicon = instance.favicon


@receiver(post_save, sender=GeneralInfoConfiguration)
def delete_favicon(sender, instance, **kwargs):
    if hasattr(instance, '_current_favicon'):
        if instance._current_favicon != instance.favicon:
            instance._current_favicon.delete(save=False)



from rest_framework import routers
from .views import AppConfiguracionViewSet, AppGeneralInfoConfigurationViewSet

router = routers.DefaultRouter()
router.register('app_configuration', AppConfiguracionViewSet, basename='configuracion')
router.register('app_configuration_general_info', AppGeneralInfoConfigurationViewSet)



# Generated by Django 3.0.5 on 2020-04-05 11:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users_extended', '0004_auto_20200404_1600'),
    ]

    operations = [
        migrations.AddField(
            model_name='extendeduser',
            name='date_of_birth',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='extendeduser',
            name='gender',
            field=models.CharField(blank=True, choices=[('M', 'Masculino'), ('F', 'Femenino')], max_length=1, null=True),
        ),
    ]



# Generated by Django 3.0.5 on 2020-04-26 02:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users_extended', '0009_auto_20200425_1410'),
    ]

    operations = [
        migrations.AddField(
            model_name='extendeduser',
            name='registration_expiration',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='extendeduser',
            name='registration_token',
            field=models.CharField(max_length=400, null=True),
        ),
    ]



from django.db import models


class Country(models.Model):
    id = models.PositiveIntegerField(primary_key=True)
    name = models.CharField(max_length=200)
    iso3 = models.CharField(max_length=3, null=True)
    iso2 = models.CharField(max_length=2, null=True)
    phone_code = models.CharField(max_length=15, null=True)
    currency = models.CharField(max_length=3, null=True)

    class Meta:
        permissions = (
            ('list_country', 'Can list countries'),
        )


class State(models.Model):
    id = models.PositiveIntegerField(primary_key=True)
    country = models.ForeignKey(Country, on_delete=models.PROTECT, related_name='states')
    name = models.CharField(max_length=200)
    state_code = models.CharField(max_length=3, null=True)

    class Meta:
        permissions = (
            ('list_state', 'Can list states'),
        )


class City(models.Model):
    id = models.PositiveIntegerField(primary_key=True)
    state = models.ForeignKey(State, on_delete=models.PROTECT, related_name='cities')
    name = models.CharField(max_length=200)

    class Meta:
        permissions = (
            ('list_city', 'Can list cities'),
        )



# Generated by Django 3.0.5 on 2020-05-03 04:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('countries', '0004_auto_20200503_0323'),
    ]

    operations = [
        migrations.AlterField(
            model_name='city',
            name='state',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='cities', to='countries.State'),
        ),
        migrations.AlterField(
            model_name='state',
            name='country',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='states', to='countries.Country'),
        ),
    ]



# Generated by Django 3.0.5 on 2020-05-04 14:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('countries', '0005_auto_20200503_0432'),
    ]

    operations = [
        migrations.AlterField(
            model_name='country',
            name='phone_code',
            field=models.CharField(max_length=15, null=True),
        ),
    ]



# Generated by Django 3.0.5 on 2020-04-11 01:32

import app_configurations.models
from django.db import migrations, models
import imagekit.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('app_configurations', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='generalinfoconfiguration',
            name='address',
            field=models.TextField(blank=True, max_length=200, null=True),
        ),
        migrations.AddField(
            model_name='generalinfoconfiguration',
            name='attention_schedule',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='generalinfoconfiguration',
            name='contact_emails',
            field=models.TextField(blank=True, max_length=200, null=True),
        ),
        migrations.AddField(
            model_name='generalinfoconfiguration',
            name='facebook',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AddField(
            model_name='generalinfoconfiguration',
            name='favicon',
            field=imagekit.models.fields.ProcessedImageField(blank=True, null=True, upload_to=app_configurations.models.GeneralInfoConfiguration.imagen_favicon_upload_to),
        ),
        migrations.AddField(
            model_name='generalinfoconfiguration',
            name='instagram',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AddField(
            model_name='generalinfoconfiguration',
            name='linkedin',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AddField(
            model_name='generalinfoconfiguration',
            name='twitter',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AddField(
            model_name='generalinfoconfiguration',
            name='whatsaap',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AddField(
            model_name='generalinfoconfiguration',
            name='youtube',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
    ]



import random

from django.db import models
from pilkit.processors import ResizeToFit
from imagekit.models import ProcessedImageField, ImageSpecField


# class UserConfiguration(models.Model):
#     regis_allowed = models.BooleanField(default=False)
#     regis_email_val = models.BooleanField(default=False)
#     regis_email_val_days = models.PositiveIntegerField(default=0)
#     rest_pass_allowed = models.BooleanField(default=False)


class GeneralInfoConfiguration(models.Model):
    application_name = models.CharField(max_length=50, default='App. Name')

    def imagen_logo_upload_to(self, filename) -> str:  # pragma: no cover
        random_number = random.randint(11111, 99999)
        return "img/logo/%s01j%sj10%s.%s" % (self.id, random_number, self.id, filename.split('.')[-1])

    def imagen_favicon_upload_to(self, filename) -> str:  # pragma: no cover
        random_number = random.randint(11111, 99999)
        return "img/favicon%s01j%sj10%s.%s" % (self.id, random_number, self.id, filename.split('.')[-1])

    favicon = ProcessedImageField(
        processors=[
            ResizeToFit(height=48, width=48, upscale=False),
        ],
        format='PNG',
        options={'quality': 100},
        null=True,
        blank=True,
        upload_to=imagen_favicon_upload_to
    )

    favicon_medium = ImageSpecField(
        source='favicon',
        processors=[
            ResizeToFit(height=32, width=32, upscale=False),
        ],
        format='PNG'
    )

    favicon_small = ImageSpecField(
        source='favicon',
        processors=[
            ResizeToFit(height=16, width=16, upscale=False),
        ],
        format='PNG'
    )

    logo = ProcessedImageField(
        processors=[
            ResizeToFit(height=500, width=500, upscale=False),
        ],
        format='PNG',
        options={'quality': 100},
        null=True,
        blank=True,
        upload_to=imagen_logo_upload_to
    )

    icon_small = ImageSpecField(
        source='logo',
        processors=[
            ResizeToFit(height=32, width=32, upscale=False),
        ],
        format='PNG'
    )

    icon_medium = ImageSpecField(
        source='logo',
        processors=[
            ResizeToFit(height=48, width=48, upscale=False),
        ],
        format='PNG'
    )

    logo_small = ImageSpecField(
        source='logo',
        processors=[
            ResizeToFit(height=256, width=256, upscale=False),
        ],
        format='PNG'
    )

    logo_medium = ImageSpecField(
        source='logo',
        processors=[
            ResizeToFit(height=300, width=300, upscale=False),
        ],
        format='PNG'
    )

    address = models.TextField(max_length=200, null=True, blank=True)
    contact_emails = models.TextField(max_length=200, null=True, blank=True)
    attention_schedule = models.TextField(null=True, blank=True)
    facebook = models.CharField(max_length=200, null=True, blank=True)
    whatsaap = models.CharField(max_length=200, null=True, blank=True)
    instagram = models.CharField(max_length=200, null=True, blank=True)
    linkedin = models.CharField(max_length=200, null=True, blank=True)
    youtube = models.CharField(max_length=200, null=True, blank=True)
    twitter = models.CharField(max_length=200, null=True, blank=True)



# Generated by Django 3.0.5 on 2020-05-03 03:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('countries', '0003_auto_20200503_0312'),
    ]

    operations = [
        migrations.AlterField(
            model_name='country',
            name='currency',
            field=models.CharField(max_length=3, null=True),
        ),
        migrations.AlterField(
            model_name='country',
            name='iso2',
            field=models.CharField(max_length=2, null=True),
        ),
        migrations.AlterField(
            model_name='country',
            name='iso3',
            field=models.CharField(max_length=3, null=True),
        ),
        migrations.AlterField(
            model_name='country',
            name='phone_code',
            field=models.PositiveIntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='state',
            name='state_code',
            field=models.CharField(max_length=3, null=True),
        ),
    ]



# Generated by Django 3.0.5 on 2020-04-04 16:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users_extended', '0003_useraditionalpermission'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='useraditionalpermission',
            options={'managed': False, 'permissions': (('list_user', 'Can list user'), ('change_password_user', 'Can change user password'), ('make_superuser_user', 'Can make user superuser'), ('make_staff_user', 'Can make user staff'), ('make_active_user', 'Can make user active'))},
        ),
    ]



import json

from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from utils.functions.functions import search_multiple_fields
from .models import Country, State, City
from .serializers import (
    CountrySerializer,
    CountryWithDetailSerializer,
    StateSerializer,
    StateWithDetailSerializer,
    CitySerializer,
    CityWithDetailSerializer,
)


class CountryViewSet(viewsets.ModelViewSet):
    serializer_class = CountrySerializer
    queryset = Country.objects.prefetch_related('states').all()

    @action(detail=False, methods=['post'])
    def create_multiple_countries(self, request):
        from .services import create_multiple_countries
        countries = json.loads(request.POST.get('countries'))
        create_multiple_countries(
            countries=countries
        )
        serializer = self.get_serializer(self.queryset, many=True)
        return Response(serializer.data)

    @action(detail=True, methods=['post'])
    def create_multiple_states_by_country(self, request, pk=None):
        from .services import create_multiple_states
        states = json.loads(request.POST.get('states'))
        create_multiple_states(
            states=states
        )
        self.serializer_class = CountryWithDetailSerializer

    @action(detail=False, methods=['post'])
    def create_multiple_states(self, request):
        from .services import create_multiple_states
        states = json.loads(request.POST.get('states'))
        create_multiple_states(
            states=states
        )
        return Response({'result': 'Se han creado los Departamentos/Estados con éxito'})

    @action(detail=False, methods=['post'])
    def create_multiple_cities(self, request):
        from .services import create_multiple_cities
        cities = json.loads(request.POST.get('cities'))
        create_multiple_cities(
            cities=cities
        )
        return Response({'result': 'Se han creado las ciudades con éxito'})

    def retrieve(self, request, *args, **kwargs):
        self.serializer_class = CountryWithDetailSerializer
        return super().retrieve(request, *args, **kwargs)


class StateViewSet(viewsets.ModelViewSet):
    serializer_class = StateSerializer
    queryset = State.objects.select_related('country').all()

    @action(detail=True, methods=['post'])
    def create_multiple_cities(self, request, pk=None):
        from .services import create_multiple_cities
        cities = json.loads(request.POST.get('cities'))
        create_multiple_cities(
            cities=cities
        )
        self.serializer_class = StateWithDetailSerializer
        serializer = self.get_serializer(self.get_object())
        return Response(serializer.data)

    def retrieve(self, request, *args, **kwargs):
        self.serializer_class = StateWithDetailSerializer
        return super().retrieve(request, *args, **kwargs)


class CityViewSet(viewsets.ModelViewSet):
    serializer_class = CitySerializer
    queryset = City.objects.all()

    def retrieve(self, request, *args, **kwargs):
        self.serializer_class = CityWithDetailSerializer
        return super().retrieve(request, *args, **kwargs)

    @action(detail=False, methods=['get'])
    def by_name(self, request):
        name = request.GET.get('name')
        qs = None
        search_fields = ['state__country__iso3', 'state__country__name', 'state__name', 'name']
        self.serializer_class = CityWithDetailSerializer

        if len(name) > 2:
            qs = search_multiple_fields(self.queryset.select_related('state', 'state__country'), search_fields, name)
        serializer = self.get_serializer(qs, many=True)
        return Response(serializer.data)



from django.contrib.auth import authenticate
from django.contrib.auth.models import User
from rest_framework import serializers


# from knox.models import AuthToken


# class AuthTokenSerializer(serializers.Serializer):
#     class Meta:
#         model = AuthToken
#         fields = [
#             'created',
#             'expiry',
#             'digest',
#         ]


class UserSerializer(serializers.ModelSerializer):
    to_string = serializers.SerializerMethodField()
    profile_image_url = serializers.SerializerMethodField()
    security_pin = serializers.CharField(source='extended_user.security_pin', allow_null=True, required=False)
    security_qr = serializers.CharField(source='extended_user.security_qr', allow_null=True, required=False)
    city_name = serializers.CharField(source='extended_user.city.name', read_only=True)
    city = serializers.IntegerField(source='extended_user.city_id', allow_null=True, required=False)
    alias = serializers.CharField(source='extended_user.alias', allow_null=True, required=False)
    gender = serializers.CharField(source='extended_user.gender', allow_null=True, required=False)
    registration = serializers.BooleanField(default=False, allow_null=True, required=False)
    # encrypted = serializers.BooleanField(default=False, allow_null=True, required=False)
    multi_sessions = serializers.NullBooleanField(
        source='extended_user.multi_sessions',
        default=False
    )
    multi_sessions_qty = serializers.IntegerField(
        source='extended_user.multi_sessions_qty',
        allow_null=True,
        required=False
    )
    date_of_birth = serializers.DateField(
        source='extended_user.date_of_birth',
        format="%Y-%m-%d",
        input_formats=['%Y-%m-%dT%H:%M:%S.%fZ', 'iso-8601'],
        allow_null=True,
        required=False
    )

    def get_profile_image_url(self, obj):  # pragma: no cover
        return obj.extended_user.profile_image.url if hasattr(
            obj,
            'extended_user'
        ) and obj.extended_user.profile_image else None

    def get_to_string(self, instance):  # pragma: no cover
        return ('%s %s' % (instance.first_name, instance.last_name)).title()

    def create(self, validated_data):
        from .services import create_update_user
        user = create_update_user(**validated_data)
        return user

    def update(self, instance, validated_data):
        from .services import create_update_user
        user = create_update_user(user=instance, **validated_data)
        return user

    class Meta:
        model = User
        fields = [
            'id',
            'registration',
            'first_name',
            'last_name',
            'email',
            'is_active',
            'is_staff',
            'password',
            'username',
            # 'encrypted',
            'last_login',
            'date_joined',
            'alias',
            'is_superuser',
            'groups',
            'user_permissions',
            'to_string',
            'profile_image_url',
            'security_pin',
            'city',
            'city_name',
            'security_qr',
            'multi_sessions',
            'multi_sessions_qty',
            'gender',
            'date_of_birth',
        ]
        extra_kwargs = {
            'password': {'write_only': True, 'required': False},
        }


class UserWithDetailSerializer(UserSerializer):
    pass


class UserLoginSerializer(serializers.Serializer):
    username = serializers.CharField()
    password = serializers.CharField()

    def validate(self, data):
        user = authenticate(**data)
        if user and user.is_active:
            return user
        raise serializers.ValidationError({'_error': 'El usuario y la contraseña no coinciden con ningún usuario!'})



# Generated by Django 3.0.5 on 2020-05-03 03:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('countries', '0002_auto_20200503_0312'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='city',
            options={'permissions': (('list_city', 'Can list cities'),)},
        ),
        migrations.AlterModelOptions(
            name='state',
            options={'permissions': (('list_state', 'Can list states'),)},
        ),
    ]



from django.contrib.auth.models import User
from countries.models import City
import random
from django.db import models
from imagekit.models import ProcessedImageField
from imagekit.processors import ResizeToFit


class ExtendedUser(models.Model):
    CHOICES_GENDER = (
        ('M', 'Masculino'),
        ('F', 'Femenino'),
    )

    def profile_image_upload_to(self, filename) -> str:  # pragma: no cover
        nro_random = random.randint(1111, 9999)
        return "img/users/profile/%s01j%sj10%s.%s" % (self.id, nro_random, self.id, filename.split('.')[-1])

    user = models.OneToOneField(User, on_delete=models.CASCADE, related_name='extended_user')
    # encrypted = models.BooleanField(default=False)
    alias = models.CharField(max_length=200, null=True, blank=True)
    date_of_birth = models.DateField(null=True, blank=True)
    gender = models.CharField(choices=CHOICES_GENDER, max_length=1, null=True, blank=True)
    profile_image = ProcessedImageField(
        processors=[ResizeToFit(300, 300)],
        format='PNG',
        options={'quality': 100},
        null=True,
        blank=True,
        upload_to=profile_image_upload_to
    )
    security_pin = models.CharField(max_length=300, null=True)
    security_qr = models.CharField(max_length=300, null=True)
    multi_sessions = models.BooleanField(default=False)
    multi_sessions_qty = models.PositiveIntegerField(default=0)
    registration_token = models.CharField(null=True, max_length=400)
    registration_expiration = models.DateField(null=True)
    city = models.ForeignKey(City, null=True, on_delete=models.PROTECT)

    @classmethod
    def encrypted_fields(cls):
        return ['first_name', 'last_name']


class UserAditionalPermission(models.Model):
    class Meta:
        managed = False
        permissions = (
            ('list_user', 'Can list user'),
            ('change_password_other_user', 'Can change other user password'),
            ('change_pin_other_user', 'Can change other user pin'),
            ('change_qr_other_user', 'Can change other user qr'),
            ('change_profile_other_user', 'Can change other user profile'),
            ('change_profile_image_other_user', 'Can change other user profile image'),
            ('change_profile_user', 'Can change user profile'),
            ('change_password_user', 'Can change user password'),
            ('change_pin_user', 'Can change user pin'),
            ('change_qr_user', 'Can change user qr'),
            ('change_profile_image_user', 'Can change user profile image'),
            ('change_permission_user', 'Can change user permission'),
            ('make_superuser_user', 'Can make user superuser'),
            ('make_staff_user', 'Can make user staff'),
            ('make_active_user', 'Can make user active'),
        )
